MODULE mo_aero_mem

  ! *mo_aero_mem* holds additional streams for diagnostics and storage
  !               of aerosol relevant quantities
  !
  ! Author:
  ! -------
  ! Philip Stier, MPI-MET           2001/2002
  !

  USE mo_kind,          ONLY: dp
  USE mo_linked_list,   ONLY: t_stream,  HYBRID, SURFACE, BELOWSUR, &
                              ABOVESUR10
  USE mo_memory_base,   ONLY: new_stream, add_stream_element,     &
                              default_stream_setting, add_stream_reference, AUTO
  USE mo_aero_m7,       ONLY: nmod
  USE mo_aero,          ONLY: NFILETYPE, CMODE, nemiss
  USE mo_tracer ,       ONLY: ntrac, trlist
  USE mo_time_event,    ONLY: io_time_event

  IMPLICIT NONE

  PRIVATE

  !--- Types to allow for pointer arrays for fast stream element access: ----------------------------------------

  TYPE, PUBLIC :: vmem2d
     REAL(dp), POINTER :: ptr(:,:)
  END TYPE vmem2d

  TYPE, PUBLIC :: vmem3d
     REAL(dp), POINTER :: ptr(:,:,:)
  END TYPE vmem3d

  !--- Service routines: ----------------------------------------------------------------------------------------

  PUBLIC :: construct_stream_aero  ! construct the diag stream

  PUBLIC :: construct_stream_diag  ! construct the diag stream

  PUBLIC :: construct_stream_input ! construct the input stream

  !--- Declarations for stream diag: -----------------------------------------------------------------------------

  TYPE (t_stream), PUBLIC, POINTER :: aero

  TYPE (vmem2d), PUBLIC, POINTER :: wdepinst(:)
  TYPE (vmem2d), PUBLIC, POINTER :: ddep(:)
  TYPE (vmem2d), PUBLIC, POINTER :: vdrydep(:)
  TYPE (vmem2d), PUBLIC, POINTER :: vsediment(:)
  TYPE (vmem2d), PUBLIC, POINTER :: d_burden(:)
  TYPE (vmem3d), PUBLIC :: rdry(nmod)
  TYPE (vmem3d), PUBLIC :: rwet(nmod)
  TYPE (vmem3d), PUBLIC :: densaer(nmod)

  !--- Auxiliary fields:

  REAL(dp), PUBLIC, POINTER :: smelt(:,:)
  REAL(dp), PUBLIC, POINTER :: velo10m(:,:)

  REAL(dp), PUBLIC, POINTER :: zprecip(:,:)
  REAL(dp), PUBLIC, POINTER :: zprecipinsoil(:,:)

  REAL(dp), PUBLIC, POINTER :: precipform(:,:,:)
  REAL(dp), PUBLIC, POINTER :: precipevap(:,:,:)
  REAL(dp), PUBLIC, POINTER :: uparfrac(:,:,:)
  REAL(dp), PUBLIC, POINTER :: relhum(:,:,:)

  REAL(dp), PUBLIC, POINTER :: daylength(:,:)

  !--- Declarations for stream diag:-------------------------------------------------------------------------

  TYPE (t_stream), PUBLIC, POINTER :: diag

  TYPE (vmem2d), PUBLIC, POINTER :: sed(:)
  TYPE (vmem2d), PUBLIC, POINTER :: wdep(:)
  TYPE (vmem2d), PUBLIC, POINTER :: wdep_conv(:)
  TYPE (vmem2d), PUBLIC, POINTER :: wdep_strat(:)

  REAL(dp), PUBLIC, POINTER :: d_emi_ms(:,:)  
  REAL(dp), PUBLIC, POINTER :: d_emi_mbc(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_moc(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_mdu(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_mss(:,:)

  REAL(dp), PUBLIC, POINTER :: d_emi_mssas(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_msscs(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_nssas(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_nsscs(:,:)

  REAL(dp), PUBLIC, POINTER :: d_emi_mduai(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_mduci(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_nduai(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_nduci(:,:)

  REAL(dp), PUBLIC, POINTER :: d_emi_dms_sea(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_dms_ter(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so2_wf(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so2_wf_3d(:,:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so4_wf_3d(:,:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so4_wf(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so2_ant(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so4_ant(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so2_vol(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so2_cvol(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so2_evol(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so4_cvol(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_so4_evol(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_bc_ant(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_bc_wf(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_bc_wf_3d(:,:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_bc_ag(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_oc_ant(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_oc_wf(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_oc_wf_3d(:,:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_oc_ag(:,:)
  REAL(dp), PUBLIC, POINTER :: d_emi_oc_bge(:,:)

  REAL(dp), PUBLIC, POINTER :: ustar_acrit(:,:)

  REAL(dp), PUBLIC, POINTER :: d_prod_ms4as(:,:)
  REAL(dp), PUBLIC, POINTER :: d_prod_ms4cs(:,:)
   
  REAL(dp), PUBLIC, POINTER :: d_prod_so2_dms_oh(:,:)
  REAL(dp), PUBLIC, POINTER :: d_prod_so4_dms_oh(:,:)
  REAL(dp), PUBLIC, POINTER :: d_prod_so2_dms_no3(:,:)
  REAL(dp), PUBLIC, POINTER :: d_prod_so4_so2_oh(:,:)

  REAL(dp), PUBLIC, POINTER :: d_prod_so4_gas(:,:,:)
  REAL(dp), PUBLIC, POINTER :: d_prod_so4_liq(:,:,:)

  REAL(dp), PUBLIC, POINTER :: d_cond_so4(:,:)

  REAL(dp), PUBLIC, POINTER :: d_nuc_so4(:,:)

  !--- Declarations for stream input: ----------------------------------------------------------------------------

  TYPE (t_stream), PUBLIC, POINTER :: input

  REAL(dp), PUBLIC, POINTER :: emi_dms_sea(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_dms_ter(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_wf(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_wf_3d(:,:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_ant_low(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_ant_high(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_vol(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_vol_alt(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_vol_lev(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_cvol(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_cvol_alt_low(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_cvol_alt_high(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_cvol_lev1(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_cvol_lev2(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_evol(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_evol_alt_low(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_evol_alt_high(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_evol_lev(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_evol_lev1(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_so2_evol_lev2(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_bc_ant(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_bc_wf(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_bc_wf_3d(:,:,:)
  REAL(dp), PUBLIC, POINTER :: emi_bc_ag(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_oc_ant(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_oc_wf(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_oc_wf_3d(:,:,:)
  REAL(dp), PUBLIC, POINTER :: emi_oc_ag(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_oc_bge(:,:)

  REAL(dp), PUBLIC, POINTER :: emi_du_thr(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_du_src(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_du_cla(:,:)

  REAL(dp), PUBLIC, POINTER :: emi_du_mai(:,:)
  REAL(dp), PUBLIC, POINTER :: emi_du_mci(:,:)

  REAL(dp), PUBLIC, POINTER :: vdms(:,:)

  INTEGER :: jt,      jmod

!----------------------------------------------------------------------------------------------------------------
!----------------------------------------------------------------------------------------------------------------

CONTAINS

  SUBROUTINE construct_stream_aero

    USE mo_tracer, ONLY: GAS,AEROSOLMASS,AEROSOLNUMBER

    IMPLICIT NONE

    CHARACTER(len=20) :: cunit

    ALLOCATE(wdepinst(ntrac))
    ALLOCATE(vdrydep(ntrac))
    ALLOCATE(vsediment(ntrac))

    !--- 1) Construct the aero stream: ------------------------------------------------------------------------------

    CALL new_stream (aero ,'aero',filetype=NFILETYPE)

    !--- Add standard fields for post-processing:

    CALL add_stream_reference (aero, 'geosp'   ,'g3b'   ,lpost=.TRUE.)
    CALL add_stream_reference (aero, 'lsp'     ,'sp'    ,lpost=.TRUE.)
    CALL add_stream_reference (aero, 'aps'     ,'g3b'   ,lpost=.TRUE.)    
    CALL add_stream_reference (aero, 'gboxarea','geoloc',lpost=.TRUE.)

    !--- Collect data from standard model:

    CALL add_stream_reference (aero, 'aclcv'   ,'g3b'   ,lpost=.TRUE.)

    !--- 2) Add stream elements: ------------------------------------------------------------------------------------

    CALL default_stream_setting (aero, units     = 'kg m-2 s-1',&
                                       lrerun    = .TRUE. ,     &
                                       laccu     = .TRUE. ,     &
                                       lpost     = .TRUE. ,     &
                                       leveltype = SURFACE,     &
                                       table     = 199,         &
                                       code      = AUTO         )


    !--- 2.8) Diagnostics for each tracer:

    DO jt=1, ntrac

       IF (trlist%ti(jt)%nphase==GAS .OR. trlist%ti(jt)%nphase==AEROSOLMASS) THEN
          cunit='kg m-2 s-1'
       ELSE IF (trlist%ti(jt)%nphase==AEROSOLNUMBER) THEN
          cunit='m-2 s-1'
       END IF

       !--- Wet deposition:

       IF(trlist%ti(jt)%nwetdep > 0 ) THEN
          CALL add_stream_element(aero, 'WDEPINST_'//TRIM(trlist%ti(jt)%fullname),wdepinst(jt)%ptr, units=cunit,    &
                                  longname='wet deposition',     laccu=.FALSE.                                      )
       END IF




    END DO


    !--- 3) Non accumulated diagnostic fields: ----------------------------------------------------------------------

    CALL default_stream_setting (aero, laccu     =.FALSE., &
                                       lrerun    =.FALSE.  )

    DO jt=1, ntrac

       !--- Dry deposition velocity:

       IF(trlist%ti(jt)%ndrydep > 0) THEN
          CALL add_stream_element(aero, 'VDRYDEP_'//TRIM(trlist%ti(jt)%fullname),    vdrydep(jt)%ptr,   units='m s-1', &
                                  longname='dry deposition velocity'                                                   )
       END IF

       !--- Sedimentation velocity:

       IF(trlist%ti(jt)%nsedi > 0) THEN
          CALL add_stream_element (aero, 'VSEDIMENT_'//TRIM(trlist%ti(jt)%fullname), vsediment(jt)%ptr, units='m s-1', &
                                   longname='sedimentation velocity'                                                   )
       END IF

    END DO

    !--- 4) Non accumulated auxiliary fields: -----------------------------------------------------------------------

    CALL default_stream_setting (aero,lpost =.FALSE., &
                                      lrerun=.TRUE.   )

    !--- 4.1) Auxiliary fields for dust emissions:

    CALL add_stream_element (aero, 'ZPRECIP',           zprecip,                                                    &
                                                                             units='mm'                             )

    CALL add_stream_element (aero, 'ZPRECIPINSOIL',     zprecipinsoil,                                              &
                                                        leveltype=BELOWSUR,  units='mm'                             )

    CALL add_stream_element (aero, 'DAYLENGTH',         daylength,                                                  &
                             longname='relative daylength',                  units='%'                              )

    !--- 4.2) Auxiliary fields for emission and deposition: 

    CALL add_stream_element (aero, 'SMELT',             smelt,                                                      &
                                                                             units='m s-1'                          )

    CALL default_stream_setting (aero, lpost=.TRUE.)

    CALL add_stream_element (aero, 'VELO10M',           velo10m,                                                    &
                             longname='10 m windspeed', leveltype=ABOVESUR10,units='m s-1'                          )


    !--- 4.3) Aerosol Properties:

    CALL default_stream_setting (aero, leveltype = HYBRID)

    DO jmod=1, nmod

       CALL add_stream_element (aero, 'RDRY_'//TRIM(CMODE(jmod)),    rdry(jmod)%ptr,      units='m',     &
                                longname='dry number median radius - '//TRIM(CMODE(jmod))                )

       CALL add_stream_element (aero, 'RWET_'//TRIM(CMODE(jmod)),    rwet(jmod)%ptr,      units='m',     &
                                longname='wet number median radius - '//TRIM(CMODE(jmod))                )

       CALL add_stream_element (aero, 'DENSAER_'//TRIM(CMODE(jmod)), densaer(jmod)%ptr,  units='kg m-3', &
                                longname='wet density'//TRIM(CMODE(jmod))                                )

    END DO


    !--- 4.4) Hydrological parameters:

    CALL add_stream_element (aero, 'RELHUM',            relhum,       units='%',          &
                             longname='relative humidity - ambient'                       )

    CALL add_stream_element (aero, 'PRECIPFORM',        precipform,   units='kg m-2 s-1', &
                             longname='precipitation formation rate'                      )

    CALL add_stream_element (aero, 'PRECIPEVAP',        precipevap,   units='kg m-2 s-1', &
                             longname='precipitation evaporation rate'                    )

    CALL add_stream_element (aero, 'UPARFRAC',          uparfrac,     units='1',          &
                             longname='fraction of gridbox occupied by updrafts'          )

  END SUBROUTINE construct_stream_aero

!--------------------------------------------------------------------------------------------------------------------

  SUBROUTINE construct_stream_diag

    USE mo_tracer, ONLY: GAS,AEROSOLMASS,AEROSOLNUMBER

    IMPLICIT NONE

    CHARACTER(len=20) :: cunit

    ALLOCATE(sed(ntrac))
    ALLOCATE(ddep(ntrac))
    ALLOCATE(wdep(ntrac))
    ALLOCATE(wdep_conv(ntrac))
    ALLOCATE(wdep_strat(ntrac))
    ALLOCATE(d_burden(ntrac))

    !--- 1) Construct the diag stream: ------------------------------------------------------------------------------

    CALL new_stream (diag ,'diag',filetype=NFILETYPE)

    !--- Add standard fields for post-processing:

    CALL add_stream_reference (diag, 'geosp'   ,'g3b'   ,lpost=.TRUE.)
    CALL add_stream_reference (diag, 'lsp'     ,'sp'    ,lpost=.TRUE.)
    CALL add_stream_reference (diag, 'aps'     ,'g3b'   ,lpost=.TRUE.)    
    CALL add_stream_reference (diag, 'gboxarea','geoloc',lpost=.TRUE.)

    !--- 2) Add stream elements: ------------------------------------------------------------------------------------

    CALL default_stream_setting (diag, units     = 'kg m-2 s-1',&
                                       lrerun    = .TRUE. ,     &
                                       laccu     = .TRUE. ,     &
                                       lpost     = .TRUE. ,     &
                                       leveltype = SURFACE,     &
                                       table     = 199,         &
                                       code      = AUTO         )

    !--- 2.1) Emissions:


    !--- 2.1.1) Total emission fields:

    CALL add_stream_element (diag, 'D_EMI_MS',          d_emi_ms,                                                   &
                             longname='sulfur emissions',                    units='kg(S) m-2 s-1'                  )
    CALL add_stream_element (diag, 'D_EMI_MBC',         d_emi_mbc,                                                  &
                             longname='black carbon emissions',              units='kg m-2 s-1'                     )
    CALL add_stream_element (diag, 'D_EMI_MOC',         d_emi_moc,                                                  &
                             longname='organic carbon emissions',            units='kg m-2 s-1'                     )
    CALL add_stream_element (diag, 'D_EMI_MDU',         d_emi_mdu,                                                  &
                             longname='dust emissions',                      units='kg m-2 s-1'                     )
    CALL add_stream_element (diag, 'D_EMI_MSS',         d_emi_mss,                                                  &
                             longname='sea salt emissions',                  units='kg m-2 s-1'                     )

    !--- 2.1.2) Detailed emissions for sea salt:

    CALL add_stream_element (diag, 'D_EMI_MSSAS',       d_emi_mssas,                                                &
                             longname='emission of MSSAS ',                  units='kg m-2 s-1'                     )
    CALL add_stream_element (diag, 'D_EMI_MSSCS',       d_emi_msscs,                                                &
                             longname='emission of MSSCS ',                  units='kg m-2 s-1'                     )
    CALL add_stream_element (diag, 'D_EMI_NSSAS',       d_emi_nssas,                                                &
                             longname='emission of NSSAS ',                  units='1 m-2 s-1'                      )
    CALL add_stream_element (diag, 'D_EMI_NSSCS',       d_emi_nsscs,                                                &
                             longname='emission of NSSCS ',                  units='1 m-2 s-1'                      )

    !--- 2.1.3) Detailed emissions for dust:

    CALL add_stream_element (diag, 'D_EMI_MDUAI',       d_emi_mduai,                                                &
                             longname='emission of MDUAI ',                  units='kg m-2 s-1'                     )
    CALL add_stream_element (diag, 'D_EMI_NDUAI',       d_emi_nduai,                                                &
                             longname='emission of NDUAI ',                  units='1 m-2 s-1'                      )
    CALL add_stream_element (diag, 'D_EMI_MDUCI',       d_emi_mduci,                                                &
                             longname='emission of MDUCI ',                  units='kg m-2 s-1'                     )
    CALL add_stream_element (diag, 'D_EMI_NDUCI',       d_emi_nduci,                                                &
                             longname='emission of NDUCI ',                  units='1 m-2 s-1'                      )

    !--- 2.1.4) Emissions for all other compounds:

    CALL add_stream_element (diag, 'D_EMI_DMS_SEA',     d_emi_dms_sea,                                              &
                             longname='DMS seawater emissions ',             units='kg(S) m-2 s-1'                  )

    CALL add_stream_element (diag, 'D_EMI_DMS_TER',     d_emi_dms_ter,                                              &
                             longname='DMS terrestrial emissions ',          units='kg(S) m-2 s-1'                  )

    IF (nemiss ==1 ) THEN
 
    CALL add_stream_element (diag, 'D_EMI_SO2_WF',      d_emi_so2_wf,                                               &
                             longname='SO2 wildfire emissions',              units='kg(S) m-2 s-1'                  )
    
    ELSE IF (nemiss ==2) THEN
  
    CALL add_stream_element (diag, 'D_EMI_SO2_WF_3D',   d_emi_so2_wf_3d,     leveltype=HYBRID,                      &
                             longname='SO2 wildfire emissions',              units='kg(S) m-2 s-1'                  )

    CALL add_stream_element (diag, 'D_EMI_SO4_WF_3D',   d_emi_so4_wf_3d,     leveltype=HYBRID,                      &
                             longname='primary SO4 wildfire emissions',      units='kg(S) m-2 s-1'                  )
    ELSE IF (nemiss ==3) THEN

    CALL add_stream_element (diag, 'D_EMI_SO2_WF',      d_emi_so2_wf,                                               &
                             longname='SO2 wildfire emissions',              units='kg(S) m-2 s-1'                  )

    CALL add_stream_element (diag, 'D_EMI_SO4_WF',     d_emi_so4_wf,                                                &
                             longname='primary SO4 wildfire emissions',      units='kg(S) m-2 s-1'                  )
        
    ENDIF
   
    CALL add_stream_element (diag, 'D_EMI_SO2_ANT',     d_emi_so2_ant,                                              &
                             longname='SO2 anthropogenic emissions',         units='kg(S) m-2 s-1'                  )

    IF (nemiss==2 .OR. nemiss==3) THEN

    CALL add_stream_element (diag, 'D_EMI_SO4_ANT',     d_emi_so4_ant,                                              &
                             longname='primary SO4 anthropogenic emissions', units='kg(S) m-2 s-1'                  )   
    END IF

    IF (nemiss==1) THEN

    CALL add_stream_element (diag, 'D_EMI_SO2_VOL',     d_emi_so2_vol,                                              &
                             longname='SO2 volcanic emissions',              units='kg(S) m-2 s-1'                  )

    ELSE IF (nemiss==2 .OR. nemiss==3) THEN

    CALL add_stream_element (diag, 'D_EMI_SO2_EVOL',     d_emi_so2_evol,                                            &
                             longname='SO2 expl. volcanic emissions',        units='kg(S) m-2 s-1'                  )
    
    
    CALL add_stream_element (diag, 'D_EMI_SO2_CVOL',     d_emi_so2_cvol,                                            &
                             longname='SO2 cont. volcanic emissions',        units='kg(S) m-2 s-1'                  )


    CALL add_stream_element (diag, 'D_EMI_SO4_EVOL',     d_emi_so4_evol,                                            &
                             longname='primary SO4 expl. volcanic emissions',units='kg(S) m-2 s-1'                  )
    
    
    CALL add_stream_element (diag, 'D_EMI_SO4_CVOL',     d_emi_so4_cvol,                                            &
                             longname='primary SO4 cont. volcanic emissions',units='kg(S) m-2 s-1'                  )


    END IF

    CALL add_stream_element (diag, 'D_EMI_BC_ANT',      d_emi_bc_ant,                                               &
                             longname='BC anthropogenic emissions',          units='kg m-2 s-1'                     )

    IF (nemiss ==1 .OR. nemiss==3) THEN
    
    CALL add_stream_element (diag, 'D_EMI_BC_WF',       d_emi_bc_wf,                                                &
                             longname='BC wildfire emissions',               units='kg m-2 s-1'                     )

    ELSE IF (nemiss ==2) THEN
   
    CALL add_stream_element (diag, 'D_EMI_BC_WF_3D',    d_emi_bc_wf_3d,      leveltype=HYBRID,                      &
                             longname='BC wildfire emissions',               units='kg m-2 s-1'                     )

    ENDIF

    CALL add_stream_element (diag, 'D_EMI_BC_AG',       d_emi_bc_ag,                                                &
                             longname='BC agric. waste and fuelwood emiss.', units='kg m-2 s-1'                     )

    CALL add_stream_element (diag, 'D_EMI_OC_ANT',      d_emi_oc_ant,                                               &
                             longname='OC anthropogenic emissions',          units='kg m-2 s-1'                     )

    IF (nemiss==1 .OR. nemiss==3) THEN

    CALL add_stream_element (diag, 'D_EMI_OC_WF',       d_emi_oc_wf,                                                &
                             longname='OC wildfire emissions',               units='kg m-2 s-1'                     )

    ELSE IF (nemiss==2) THEN
       
    CALL add_stream_element (diag, 'D_EMI_OC_WF_3D',    d_emi_oc_wf_3d,      leveltype=HYBRID,                      &
                             longname='OC wildfire emissions',               units='kg m-2 s-1'                     )
    
    ENDIF

    CALL add_stream_element (diag, 'D_EMI_OC_AG',       d_emi_oc_ag,                                                &
                             longname='OC agric. waste and fuelwood emiss.', units='kg m-2 s-1'                     )

    CALL add_stream_element (diag, 'D_EMI_OC_BGE',      d_emi_oc_bge,                                               &
                             longname='OC biogenic emissions',               units='kg m-2 s-1'                     )

    !--- Additional dust diagnostics:

    CALL add_stream_element (diag, 'USTAR_ACRIT',       ustar_acrit,         units='%',                             &
                             longname='time fraction with ustar > dust threshold', laccu=.TRUE.,                    &
                             contnorest=.TRUE., leveltype=SURFACE, lrerun=.TRUE.                                    )

    !--- 2.2) Mass diagnostics:

    CALL add_stream_element (diag, 'D_PROD_MS4AS',      d_prod_ms4as,                                               &
                             longname='sulfate production liquid phase acc',    units='kg(S) m-2 s-1'               )

    CALL add_stream_element (diag, 'D_PROD_MS4CS',      d_prod_ms4cs,                                               &
                             longname='sulfate production liquid phase coarse', units='kg(S) m-2 s-1'               )

    CALL add_stream_element (diag, 'D_PROD_SO2_DMS_OH', d_prod_so2_dms_oh,                                          &
                             longname='sulfur production gas phase via DMS+OH', units='kg(S) m-2 s-1'               )
    
    CALL add_stream_element (diag, 'D_PROD_SO4_DMS_OH', d_prod_so4_dms_oh,                                          &
                             longname='sulfate production gas phase via DMS+OH',units='kg(S) m-2 s-1'               )

    CALL add_stream_element (diag, 'D_PROD_SO2_DMS_NO3',d_prod_so2_dms_no3,                                         &
                             longname='sulfur production gas phase via DMS+NO3',units='kg(S) m-2 s-1'               )

    CALL add_stream_element (diag, 'D_PROD_SO4_SO2_OH', d_prod_so4_so2_oh,                                          &
                             longname='sulfate production gas phase via SO2+OH',units='kg(S) m-2 s-1'               )

    CALL add_stream_element (diag, 'D_COND_SO4',        d_cond_so4,                                                 &
                             longname='condensation of sulfate on aerosol',     units='kg(S) m-2 s-1'               )

    CALL add_stream_element (diag, 'D_NUC_SO4',         d_nuc_so4,                                                  &
                             longname='nucleation of sulfate',                  units='kg(S) m-2 s-1'               )

    !--- 3D fields:

    CALL default_stream_setting (diag, leveltype = HYBRID)

    CALL add_stream_element (diag, 'D_PROD_SO4_LIQ',    d_prod_so4_liq,                                             &
                             longname='sulfate production liquid phase',        units='kg(S) m-2 s-1'               )

    CALL add_stream_element (diag, 'D_PROD_SO4_GAS',    d_prod_so4_gas,                                             &
                             longname='sulfate production gas phase',           units='kg(S) m-2 s-1'               )


    !--- 2.3) Diagnostics for each tracer:

    CALL default_stream_setting (diag, leveltype = SURFACE)

    DO jt=1, ntrac

       IF (trlist%ti(jt)%nphase==GAS .OR. trlist%ti(jt)%nphase==AEROSOLMASS) THEN
          cunit='kg m-2 s-1'
       ELSE IF (trlist%ti(jt)%nphase==AEROSOLNUMBER) THEN
          cunit='m-2 s-1'
       END IF

       !--- 2.3.1) Sedimentation: 

       IF(trlist%ti(jt)%nsedi > 0) THEN

          CALL add_stream_element(diag, 'SED_'//TRIM(trlist%ti(jt)%fullname),     sed(jt)%ptr, units=cunit,         &
                                  longname='sedimentation'                                                          )

       END IF

       !--- 2.3.2) Wet deposition:

       IF(trlist%ti(jt)%nwetdep > 0 ) THEN

          CALL add_stream_element(diag, 'WDEP_'//TRIM(trlist%ti(jt)%fullname),      wdep(jt)%ptr,       units=cunit,&
                                  longname='wet deposition'                                                         )

          CALL add_stream_element(diag, 'WDEP_CONV_'//TRIM(trlist%ti(jt)%fullname), wdep_conv(jt)%ptr,  units=cunit,&
                                  longname='wet deposition'                                                         )

          CALL add_stream_element(diag, 'WDEP_STRAT_'//TRIM(trlist%ti(jt)%fullname),wdep_strat(jt)%ptr, units=cunit,&
                                  longname='wet deposition'                                                         )

       END IF

       !--- 2.3.3) Dry deposition:

       IF(trlist%ti(jt)%ndrydep > 0 ) THEN

          CALL add_stream_element(diag, 'DDEP_'//TRIM(trlist%ti(jt)%fullname),    ddep(jt)%ptr, units=cunit,        &
                                  longname='dry deposition'                                                         )

       END IF

       !--- 2.3.4) Atmospheric column burden:

       IF( INDEX(trlist%ti(jt)%units,'kg-1')/=0 ) THEN

          IF (trlist%ti(jt)%nphase==GAS .OR. trlist%ti(jt)%nphase==AEROSOLMASS) THEN
             cunit='kg m-2'
          ELSE IF (trlist%ti(jt)%nphase==AEROSOLNUMBER) THEN
             cunit='m-2'
          END IF

          CALL add_stream_element(diag, 'D_BURDEN_'//TRIM(trlist%ti(jt)%fullname),d_burden(jt)%ptr,units=cunit,     &
                                  longname='atmospheric burden'                                                     )

       END IF

    END DO ! jt=1, ntrac

  END SUBROUTINE construct_stream_diag

!--------------------------------------------------------------------------------------------------------------------

  SUBROUTINE construct_stream_input

    USE mo_aero, ONLY: ndust

    IMPLICIT NONE

    CHARACTER(len=20) :: unit_so2, unit_oc, unit_pom

    !--- Default output interval:

   TYPE(io_time_event),SAVE  :: input_out=io_time_event(1,'months','last',0)
!    TYPE(io_time_event),SAVE  :: input_out=io_time_event(1,'steps','last',0)

    !--- 0) Define units:

    unit_oc='kg(OC) m-2 s-1'
    unit_pom='kg(OM) m-2 s-1'
    IF (nemiss==1) THEN
       unit_so2='kg(S) m-2 s-1'    
    ELSE IF (nemiss==2 .OR. nemiss==3) THEN
       unit_so2='kg(SO2) m-2 s-1'      
    END IF
    

    !--- 1) Construct the input stream: -------------------------------------------------------------------------------

    CALL new_stream (input ,'input',filetype=NFILETYPE,interval=input_out)

    !--- Add standard fields for post-processing:

    CALL add_stream_reference (input, 'geosp'   ,'g3b'   ,lpost=.TRUE.)
    CALL add_stream_reference (input, 'lsp'     ,'sp'    ,lpost=.TRUE.)
    CALL add_stream_reference (input, 'aps'     ,'g3b'   ,lpost=.TRUE.)    
    CALL add_stream_reference (input, 'gboxarea','geoloc',lpost=.TRUE.)

    !--- 2) Add stream elements: ------------------------------------------------------------------------------------

    CALL default_stream_setting (input, units    = 'kg m-2 s-1',&
                                        lrerun   = .TRUE. ,     &
                                        laccu    = .FALSE. ,    &
                                        lpost    = .TRUE. ,     &
                                        leveltype = SURFACE,    &
                                        code     = AUTO         )

    !--- 2.1) Prescribed fields used for emissions:

    CALL add_stream_element (input, 'EMI_DMS_SEA',     emi_dms_sea,                                              &
                             longname='DMS seawater concentration ',         units='nmole l-1'                   )

    CALL add_stream_element (input, 'EMI_DMS_TER',     emi_dms_ter,                                              &
                             longname='DMS terrestrial emissions ',          units='kg(S) m-2 s-1'               )

    IF (nemiss==1 .OR. nemiss==3) THEN

    CALL add_stream_element (input, 'EMI_SO2_WF',      emi_so2_wf,                                               &
                             longname='SO2 wildfire emissions',              units=TRIM(unit_so2)                )

    ELSE IF (nemiss==2) THEN

    CALL add_stream_element (input, 'EMI_SO2_WF_3D',   emi_so2_wf_3d,        leveltype=HYBRID,                   &
                             longname='SO2 wildfire emissions',              units=TRIM(unit_so2)                )

    END IF 

    CALL add_stream_element (input, 'EMI_SO2_ANT_LOW', emi_so2_ant_low,                                          &
                             longname='SO2 low level anthrop. emissions',    units=TRIM(unit_so2)                )

    CALL add_stream_element (input, 'EMI_SO2_ANT_HIGH',emi_so2_ant_high,                                         &
                             longname='SO2 high level anthrop. emissions',   units=TRIM(unit_so2)                )
    
    IF (nemiss==1) THEN

    CALL add_stream_element (input, 'EMI_SO2_VOL',     emi_so2_vol,                                              &
                             longname='SO2 volcanic emissions',              units=TRIM(unit_so2)                )
    
    CALL add_stream_element (input, 'EMI_SO2_VOL_ALT', emi_so2_vol_alt,                                          &
                             longname='SO2 volcanic emissions - altitude',   units='m'                           )

    CALL add_stream_element (input, 'EMI_SO2_VOL_LEV', emi_so2_vol_lev,                                          &
                             longname='SO2 volcanic emissions - level',      units='model level'                 )

    ELSE IF (nemiss==2 .OR. nemiss==3) THEN

    CALL add_stream_element (input, 'EMI_SO2_EVOL',          emi_so2_evol,                                       &
                             longname='SO2 expl. volcanic emissions',              units=TRIM(unit_so2)          )    

    CALL add_stream_element (input, 'EMI_SO2_EVOL_ALT_HIGH', emi_so2_evol_alt_high,                              &
                             longname='SO2 expl. volcanic emissions - altitude high',   units='m'                )

    CALL add_stream_element (input, 'EMI_SO2_EVOL_ALT_LOW',  emi_so2_evol_alt_low,                               &
                             longname='SO2 expl. volcanic emissions - altitude low',   units='m'                 )
    
    CALL add_stream_element (input, 'EMI_SO2_EVOL_LEV1',      emi_so2_evol_lev1,                                 &
                             longname='SO2 expl. volcanic emissions - level low',       units='model level'      )

    CALL add_stream_element (input, 'EMI_SO2_EVOL_LEV2',      emi_so2_evol_lev2,                                 &
                             longname='SO2 expl. volcanic emissions - level high',      units='model level'      )
    
    CALL add_stream_element (input, 'EMI_SO2_CVOL',          emi_so2_cvol,                                       &
                             longname='SO2 cont. volcanic emissions',              units=TRIM(unit_so2)          )    

    CALL add_stream_element (input, 'EMI_SO2_CVOL_ALT_HIGH', emi_so2_cvol_alt_high,                              &
                             longname='SO2 cont. volcanic emissions - altitude high',   units='m'                )

    CALL add_stream_element (input, 'EMI_SO2_CVOL_ALT_LOW',  emi_so2_cvol_alt_low,                               &
                             longname='SO2 cont. volcanic emissions - altitude low',   units='m'                 )
    
    CALL add_stream_element (input, 'EMI_SO2_CVOL_LEV1',      emi_so2_cvol_lev1,                                 &
                             longname='SO2 cont. volcanic emissions - level high', units='model level'           )
    
    CALL add_stream_element (input, 'EMI_SO2_CVOL_LEV2',      emi_so2_cvol_lev2,                                 &
                             longname='SO2 cont. volcanic emissions - level low',  units='model level'           )
    
    END IF

    CALL add_stream_element (input, 'EMI_BC_ANT',      emi_bc_ant,                                               &
                             longname='BC anthropogenic emissions',          units='kg m-2 s-1'                  )
    IF (nemiss==1 .OR. nemiss==3) THEN

    CALL add_stream_element (input, 'EMI_BC_WF',       emi_bc_wf,                                                &
                             longname='BC wildfire emissions',               units='kg m-2 s-1'                  )

    ELSE IF (nemiss==2) THEN
   
    CALL add_stream_element (input, 'EMI_BC_WF_3D',    emi_bc_wf_3d,         leveltype=HYBRID,                   &
                             longname='BC wildfire emissions',               units='kg m-2 s-1'                  )

    END IF

    CALL add_stream_element (input, 'EMI_BC_AG',       emi_bc_ag,                                                &
                             longname='BC agric. waste and fuelwood emiss.', units='kg m-2 s-1'                  )

    CALL add_stream_element (input, 'EMI_OC_ANT',      emi_oc_ant,                                               &
                             longname='BC anthropogenic emissions',          units=TRIM(unit_oc)                 )
    IF (nemiss==1 .OR. nemiss==3) THEN

    CALL add_stream_element (input, 'EMI_OC_WF',       emi_oc_wf,                                                &
                             longname='OC wildfire emissions',               units=TRIM(unit_oc)                 )
    ELSE IF (nemiss==2) THEN
 
    CALL add_stream_element (input, 'EMI_OC_WF_3D',    emi_oc_wf_3d,         leveltype=HYBRID,                   &
                             longname='OC wildfire emissions',               units=TRIM(unit_pom)                )

    END IF

    CALL add_stream_element (input, 'EMI_OC_AG',       emi_oc_ag,                                                &
                             longname='OC agric. waste and fuelwood emiss.', units=TRIM(unit_oc)                 )

    IF (nemiss ==1 .OR. nemiss ==3) THEN

    CALL add_stream_element (input, 'EMI_OC_BGE',      emi_oc_bge,                                               &
                             longname='OC biogenic emissions',               units=TRIM(unit_oc)                 )

    ELSE IF (nemiss==2) THEN

    CALL add_stream_element (input, 'EMI_OC_BGE',      emi_oc_bge,                                               &
                             longname='OC biogenic emissions',               units=TRIM(unit_pom)                )
    END IF

    IF (ndust==1) THEN

    CALL add_stream_element (input, 'EMI_DU_THR',      emi_du_thr,                                               &
                             longname='DU emissions; threshold velocity',    units='m s-1'                       )

    CALL add_stream_element (input, 'EMI_DU_SRC',      emi_du_src,                                               &
                             longname='DU emissions; source strength factor',units='kg s+2 m-5'                  )

    CALL add_stream_element (input, 'EMI_DU_CLA',      emi_du_cla,                                               &
                             longname='DU emissions; clay fraction in soil', units='%'                           )
    
    ELSE IF (ndust==2) THEN
    
    CALL add_stream_element (input, 'EMI_DU_MAI',      emi_du_mai,                                               &
                             longname='DU emissions accumulation mode ',     units='kg m-2 s-1'                  )

    CALL add_stream_element (input, 'EMI_DU_MCI',      emi_du_mci,                                               &
                             longname='DU emissions coarse mode ',           units='kg m-2 s-1'                  )

    ENDIF

    !--- 2.2) Calculated piston velocity for DMS:

    CALL add_stream_element (input, 'VDMS',            vdms,                                                     &
                             longname='piston velocity dms',                 units='cm hr-1'                     )


  END SUBROUTINE construct_stream_input

!--------------------------------------------------------------------------------------------------------------------

END MODULE mo_aero_mem
