MODULE mo_debugs
  !-----------------------------------------------------------------------
  ! Module containing variables in order to debug mozech
  !
  ! Authors:
  ! J.S. Rast, MPI, September 2003, original source
  !-----------------------------------------------------------------------

  USE mo_kind,                    ONLY: dp
  USE mo_memory_base,             ONLY: new_stream, add_stream_element, &
                                        default_stream_setting, AUTO, t_stream
  USE mo_time_event,              ONLY: io_time_event
  USE mo_time_control,            ONLY: p_bcast_event

  IMPLICIT NONE

  PRIVATE
! debugs stream
  PUBLIC :: init_debugs
  PUBLIC :: ddf1, ddf2, ddf3, ddf4, ddf5, ddf6, &
            ddf7, ddf8, ddf9,ddf10,ddf11,ddf12
  PUBLIC :: zdf1, zdf2, zdf3, zdf4, zdf5, zdf6, &
            zdf7, zdf8, zdf9,zdf10,zdf11,zdf12
  PUBLIC :: nddf, nzdf

  INTEGER           :: nddf=0, nzdf=0
  REAL(dp), POINTER :: ddf1(:,:,:), ddf2(:,:,:), ddf3(:,:,:), ddf4(:,:,:), &
                       ddf5(:,:,:), ddf6(:,:,:), ddf7(:,:,:), ddf8(:,:,:), &
                       ddf9(:,:,:),ddf10(:,:,:),ddf11(:,:,:),ddf12(:,:,:)
  REAL(dp), POINTER :: zdf1(:,:), zdf2(:,:), zdf3(:,:), zdf4(:,:), &
                       zdf5(:,:), zdf6(:,:), zdf7(:,:), zdf8(:,:), &
                       zdf9(:,:),zdf10(:,:),zdf11(:,:),zdf12(:,:)
  TYPE(io_time_event), SAVE     :: putdebug_stream
  CONTAINS


  SUBROUTINE init_debugs

    USE mo_mpi,                   ONLY: p_parallel_io, p_barrier, p_bcast, p_io
    USE mo_namelist,              ONLY: position_nml, nnml, POSITIONED

    CHARACTER(LEN=32) :: ichar
    INTEGER           :: i, ierr
    REAL(dp), POINTER :: vddf(:,:,:), vzdf(:,:)
    TYPE (t_stream), POINTER      :: debugs

    INCLUDE 'debugsctl.inc'
    
! set default output interval
    putdebug_stream%counter      = 6
    putdebug_stream%unit         = 'hours'
    putdebug_stream%adjustment   = 'first'
    putdebug_stream%offset       = 0
    IF (p_parallel_io) THEN
       CALL position_nml ('DEBUGSCTL', status=ierr)
       SELECT CASE (ierr)
          CASE (POSITIONED)
             READ(nnml, debugsctl)
       END SELECT
    END IF
    CALL p_barrier
    CALL p_bcast (nddf, p_io)
    CALL p_bcast (nzdf, p_io)
    CALL p_bcast_event (putdebug_stream, p_io)
    CALL new_stream (debugs,'debugs',lrerun=.false.,interval=putdebug_stream)
    CALL default_stream_setting (debugs,lrerun=.false.,contnorest=.true., &
                                 laccu=.false.,lpost=.true.,table=199, &
                                 code=AUTO)
    CALL add_stream_element (debugs,'ddf1',ddf1)
    CALL add_stream_element (debugs,'ddf2',ddf2)
    CALL add_stream_element (debugs,'ddf3',ddf3)
    CALL add_stream_element (debugs,'ddf4',ddf4)
    CALL add_stream_element (debugs,'ddf5',ddf5)
    CALL add_stream_element (debugs,'ddf6',ddf6)
    CALL add_stream_element (debugs,'ddf7',ddf7)
    CALL add_stream_element (debugs,'ddf8',ddf8)
    CALL add_stream_element (debugs,'ddf9',ddf9)
    CALL add_stream_element (debugs,'ddf10',ddf10)
    CALL add_stream_element (debugs,'ddf11',ddf11)
    CALL add_stream_element (debugs,'ddf12',ddf12)

    CALL add_stream_element (debugs,'zdf1',zdf1)
    CALL add_stream_element (debugs,'zdf2',zdf2)
    CALL add_stream_element (debugs,'zdf3',zdf3)
    CALL add_stream_element (debugs,'zdf4',zdf4)
    CALL add_stream_element (debugs,'zdf5',zdf5)
    CALL add_stream_element (debugs,'zdf6',zdf6)
    CALL add_stream_element (debugs,'zdf7',zdf7)
    CALL add_stream_element (debugs,'zdf8',zdf8)
    CALL add_stream_element (debugs,'zdf9',zdf9)
    CALL add_stream_element (debugs,'zdf10',zdf10)
    CALL add_stream_element (debugs,'zdf11',zdf11)
    CALL add_stream_element (debugs,'zdf12',zdf12)

    DO i=1,nddf
       WRITE(ichar,*) i
       CALL add_stream_element (debugs,'vddf'//TRIM(ADJUSTL(ichar)),vddf)
    END DO
    DO i=1,nzdf
       WRITE(ichar,*) i
       CALL add_stream_element (debugs,'vzdf'//TRIM(ADJUSTL(ichar)),vzdf)
    END DO
  END SUBROUTINE init_debugs
  END MODULE mo_debugs
